# Templates

The templating system in Kado relies upon `mustache` and `mustache-express` to learn about mustache [see here](https://mustache.github.io/mustache.5.html)

Each interface in Kado shares a similar set of global template variables and then contextual variables will vary with module and page. It is important to note that global variables are denounced with `_something` underscore(`_`) specifying that this is a global variable and `something` being the name.

## Global Template Variables


| Name | Description | Default | Notes |
| --- | --- | --- | --- |
| \_basedir | The base folder where the interface views are stored. | `app.get('views')` |  |
| \_moment | The moment.js date library | `require('moment')` |  |
| \_timezone | Shortcut tot he current timezone | n/a | The users timezone |
| \_printDate | Function to print a date in a standard formate | `function` | Usage: `{{#_printDate}}{{someDate}}{{/_printDate}}` |
| \_toDateString | Alias for `_printDate` | n/a | n/a |
| \_escapeAndTruncate | Strip HTML and special chars then truncate to length. | `function` | Usage: `{{#_escapeAndTruncate}}50,{{someVar}}{{/_escapeAndTruncate}}` |
| \_is | Conditionally print a variable | `function` | Usage: `{{#_is}}true,one,two{{/_is}}` Will print `one` because `true` is the first arg, whereas `false` would cause `two` to print. Commmas in the input will cause this function to error. |
| \_compare | Conditionally print a variable by comparing two inputs | `function` | Usage: `{{#_compare}}one,two,three,four{{/_compare}}`. This example will print `four` because `one` does not match `two`. Commas within the input will cause this function to error. |
| \_capitalize | Capitalize the first letter of words in a string | `function` | Usage: `{{#_capitalize}}some words{{/_capitalize}}` Will print `Some Words` |
| \_prettyBytes | Access to the `prettyBytes` library | `require('pretty-bytes')` |  |
| \_appName | Current application name | kado |  |
| \_appTitle | Current application title | Kado |  |
| \_version | Current Kado version | n/a |  |
| \_currentYear | Current year in YYYY format | n/a |  |
| \_breadcrumb | Access to `app.breadcrumb` to load breadcrumb system | `app.breadcrumb` |  |
| \_c | Compiled content partial access. | `Object` | Usage: `{{_c.some_partial}}` NOTE: The content manager the partial must be named `partial_some_partial`. |
| \_nav | Access to `app.nav` to load navigation system | `app.nav` |  |
| \_l | Compiled language pack set for the page | `Object` | Usage: `{{_l.some_module.some_word}}` Will display the phrase from the language pack or nothing. |
| \_localNav | Local navigation built from the Content Navigation module | `Object` |  |
| \_permission | Access to `app.permission` to validate permissions | `app.permission` |  |
| \_p.show | Show content based on permission | `function` | Usage: `{{#_p.show}}some-permission,{{someVar}}{{/_p.show}}` This will only show the `{{someVar}}` when the permission is available to the current user. Commas within the input will cause this function to error. |
| \_uri | Access to `app.uri` to load uri system | `app.uri` |  |
| \_u | Compiled URI structure for the system | `Object` | All registered URIs will be available through this object as `{{_u._some_uri_registered_in_the_system}}` all slashes within the URI will be replaced with underscores. |
| \_view | Access to `app.view` to load the view system | `app.view` |  |
| \_currentUri | The current URI being viewed | n/a | Useful for activating navigation links |
| \_pageTitle | The page title generated by Kado or modules | n/a | Dynamic page titles which is set by the module or defaults within the system |
| \_searchPhrase | Current search phrase being used by the user. | null |  |
